package builder;

import java.awt.*;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.io.*;



/**	An Abstraction.  Represents an Object that has an Icon representation
	which can be used to inform a Map how to handle the information that it 
	contains.  It may also have a GUI representation that appears inside a
	Form and either accepts input from the user or presents results.
	May be turned into an Abstract Class.
	* @author Arturo Falck
	* @author Dennis Seay
	* @version 1.1 07/97
*/
public class Node extends Moveable{



// ********************* CONSTRUCTORS


/**	Main Constructor.
*/
	Node (String text, Map map, Point locMap, 
			Map form, Point locForm) {
		super();
		nodeNumber 	= map.addNode(this);	// add NODE to Vector.
		setMap(form);
		setLocMap(locForm);
		setListener(this);
		initialize(Node.EDIT_COLOR, new Dimension(123, 33));

		setIcon(text, map, locMap);

		Point initialLabelPosition = 
			new Point( 	getLocMapX () + 5 ,
					getLocMapY () + this.getBounds().height );
		formLabel = new Name (text, getMap(), initialLabelPosition);


	}


// ********************* METHODS




/**	getNodeNumber
	* @return the index of this Node.
*/
	public int getNodeNumber(){
		return nodeNumber;
	}


/**	getFormLabel
	* @return a Name associated with the GUI representation of this node.
*/
	public Name getFormLabel(){
		return formLabel;
	}

/**	lockFormLabel
	Locks the Label for this Node in place or unlocks it.
	* @param lock true if locked, false if unlocked.
*/
	public void lockFormLabel(boolean lock){
		formLabel.lock(lock);			
	}


/**	getIoOption
	* @return the state of this node.
		Node.INPUT for input.
		Node.OUTPUT for output.
*/
	public char getIoOption(){
		return ioOption;
	}

/**	setIoOption
	Sets the state of this node.
	* @param option 
		Node.INPUT for input.
		Node.OUTPUT for output.
*/
	public void setIoOption(char option){
		ioOption = option;
		Icon temp = icon;
		if (ioOption == INPUT) 
			icon = new InputIcon(	temp.getIconLabel().toString(), 
						temp.getMap(),
						temp.getLocMap(),
						temp.getParentNode());
		if (ioOption == OUTPUT) 
			icon = new OutputIcon(	temp.getIconLabel().toString(), 
						temp.getMap(),
						temp.getLocMap(),
						temp.getParentNode());
		temp.removeIcon();
	}



/**	getIcon
	* @return the Icon associated with this node.
*/
	public Icon getIcon(){
		return icon;
	}

/**	setIcon
	Associates an Icon to this node.
	* @param text the label associated with the icon.
	* @param map the map where the icon will be located.
	* @param locMap where in the map to locate the icon initially.
*/
	public void setIcon (String text, Map map, Point locMap){
	
		if (getIoOption() == INPUT) 
			icon = new InputIcon(text, map, locMap, this);
		else if (getIoOption() == OUTPUT) 
			icon = new OutputIcon(text, map, locMap, this);
		else  
		 	icon = new Icon(text, map, locMap, this);

	}

/**	setIcon
	Associates an Icon to this node.
	* @param icon a Moveable Icon.
*/
	public void setIcon (Icon icon){
		this.icon = icon;
	}
/** setListener
  * This function sets the mouse and moveable mouse listeners
  * @param obj is the moveable object that is the parent for the listeners
*/
  public void setListener(Moveable obj)
  {
    addMouseListener(new NodeMouseListener(obj));
    addMouseMotionListener(new MoveableMouseMotionListener(obj));
  }

/**	moveIcon
	Moves the Icon within the map.
	* @param delta how much to move the icon from its present position.
*/
	public void moveIcon(Point delta){
		icon.move(delta);
	}


/**	setMoving
	sets the it's own moving status and the moving status of it's label.
	* @param status true if moving, flase if not moving.
*/
	public void setMoving(boolean status) {
		formLabel.setMoving(status);
		formLabel.lock(status);
		super.setMoving(status);
	}

/**	move
	Moves the GUI representation of this node within the Form.
	* @param delta how much to move the place-holder 
	from its present position.
*/
	public void move(Point delta){
		formLabel.move(delta);
		super.move(delta);
	}


/**	addWire
	Adds Wire to the vector.
	* @return the index of the wire.
*/
	public int addWire(Wire wire){
		wiresVector.addElement(wire);
		return wiresVector.size();
	}


/**	removeWire
	removes Wire from the vector.
*/
  public void removeWire(Wire wire)
  {
    wiresVector.removeElement(wire);
  }


/**	addComponent
	Adds the Component to this Node.
	* @param c the component.
*/
	public void addComponent(Component c){
		setBackground(getMap().getBackground());
		setPaint(false);
		c.setBounds(	0,
				0,
				this.getBounds().width,
				this.getBounds().height	);
		add(c);
	}

/**	removeComponent
	Removes the Component to this Node.
	* @param c the component.
*/
	public void removeComponent(Component c){
		setPaint(true);
		setBackground(Node.EDIT_COLOR);
		remove(c);
	}

/**	getComponent
	* @return  the Component of this Node.  
	Must be overriden by every Node Type.
	* @param c the component.
*/
	public Component getComponent(){
		return this;
	}



/**	editMode
	Changes the GUI representation of this node from non-Editable (run mode)
	to Editable (edit mode).
*/
	public void editMode(){
		if ((getComponentAt(1,1) instanceof Node) == false) {
			lock(false);
			lockFormLabel(false);
			removeComponent(getComponent());
		}
	}

/**	runMode
	Changes the GUI representation of this node from Editable (edit mode)
	to non-Editable (run mode).
	* @return
*/
	public boolean runMode(){
		if ((getComponentAt(1,1) instanceof Node) == true) {
			lock(true);
			lockFormLabel(true);
			addComponent(getComponent());
			return true;
		}
		else
			return false;
	}


/**	paint
	Defines the message for the place-holder of this Node.
	* @param g the graphics context.
*/
	public void paint (Graphics g) {
		if (getPaint()){
			paintPlaceHolder(g);
		}
	}

/**	paintPlaceHolder
	Defines the message for the place-holder of this Node.
	* @param g the graphics context.
*/
	public void paintPlaceHolder (Graphics g) {
		g.setColor(Color.red);
		g.drawString("<<component>>", 10, 20);
		g.draw3DRect(1, 1, 120, 30, true);
	}

/**	setPaint
	Sets the status of the paint method used.
	* @param status true if PlaceHolder is painted, false otherwise.
*/
	public void setPaint (boolean status) {
		paintMode = status;
	}

/**	getPaint
	* @return true if PlaceHolder should be painted, false otherwise.
*/
	public boolean getPaint () {
		return paintMode;
	}



/**	setAttributes
	Instantiates an Object that allows the builder to change the attributes 
	of this Node. And places it next to the GUI representation of the Node.
*/
	public void setAttributes(){
		try{
			System.out.println("Node change attributes");
			ChangeAttributes c1 = new ChangeAttributes(this);
			c1.setLocation(getLocMapX() + this.getBounds().width /2,
				getLocMapY() + this.getBounds().height/2);


			getMap().add(c1);

			getMap().setVisible(true);
		}
		catch (NullPointerException e){
			System.out.println("Null Pointer");
		}
	}


/**	getArgument
	* @return the argument for this Node.
*/
	public Object getArgument(){
		return argument;
	}

/**	incommingAction
	Must be overriden.
	* @return true.
*/
	public boolean incommingAction(Node caller){
		return true;
	}

/** save
  * This function writes node information that is in required for all nodes
  * @param os is the the output stream to write the data to.
  * @param type is the type of node being saved.
*/
  public void save(PrintWriter os, String type)
  {
    os.println(type);
    os.println(getLocMapX() + " " + getLocMapY());
    os.println(formLabel.getLabel());
    os.println(formLabel.getLocMapX() + " " + formLabel.getLocMapY());
    os.println(formLabel.isVisible());
    os.println(getIoOption());
    os.println(icon.getLocMapX() + " " +icon.getLocMapY());
    os.println(((Name)icon.getIconLabel()).getLocMapX() + " " + ((Name)icon.getIconLabel()).getLocMapY());
  } 

// ********************* Event METHODS

/** nodeAction
  * this function handles Node events
  * @param arg is the node that generated the event
*/
  public void nodeAction(String arg)
  {
    argument = arg;
    if (wiresVector.isEmpty() != true)
    {
      for (int i=0; i<wiresVector.size(); i++)
      {
        ((Wire)wiresVector.elementAt(i)).transmit();
      }
    }
  }

/** rightMouseClick
  * This function brings up an attribute box. It is called when the right 
  * mouse button is clicked.
  * @param x this is the x location to put the pulldown box
  * @param y this is the y location to put the pulldown box
*/
  public void rightMouseClick(int x, int y)
  {
    try
    {
      ChangeAttributes c =
      new ChangeAttributes(this);
      c.setLocation(  getLocMapX() +
        this.getBounds().width /2,
        getLocMapY() +
        this.getBounds().height/2);
      getMap().add(c);
      getMap().setVisible(true);
    }
    catch (NullPointerException e){}
  }

// ********************* FIELDS


		Vector		wiresVector = new Vector();
	public final static String END_OF_NODE = "<--END-NODE-->";
	public final static String BEGIN_NODE_DATA = "<--BEGIN-NODE-DATA-->";
	public final static String END_NODE_DATA = "<--END-NODE-DATA-->";
	public final static Color EDIT_COLOR = Color.pink;
	public final static char INPUT  = 'i';
	public final static char OUTPUT = 'o';

	private int 		nodeNumber = 0;
	private char 		ioOption = INPUT;
	private Icon 		icon;
	private Name 		formLabel;
		Object 		argument;
	private boolean		paintMode = true;
}
